"""Token"""
import json
from typing import Any

from loguru import logger

from .cipher import AESCipher
from .utils import datetime_now, datetime_offset, datetime_string_to_datetime, datetime_to_string, v_true


def generate_token(key: str = 'Fc0zXCmGKd7tPu6W', timeout: int = 3600, data: Any = None) -> (str | None):
    try:
        now = datetime_now()

        if now is None:
            return None

        offset = datetime_offset(now, seconds=+timeout)

        if offset is None:
            return None

        source = json.dumps(
            obj={
                "datetime": datetime_to_string(offset),
                "data": data
            },
            default=str
        )

        cipher = AESCipher(key=key, algorithm='sha256')

        return cipher.encrypt(source)

    except Exception as e:
        logger.exception(e)
        return None


def parsing_token(token_string: str, key: str = 'Fc0zXCmGKd7tPu6W') -> (dict | None):
    try:
        if v_true(token_string, str) is False:
            return None

        cipher = AESCipher(key=key, algorithm='sha256')

        target = cipher.decrypt(token_string)

        if target is None:
            return None

        source: dict = json.loads(target)

        source['datetime'] = datetime_string_to_datetime(source['datetime'])

        return source

    except Exception as e:
        logger.exception(e)
        return None


def certify_token(token_string: str, key: str = 'Fc0zXCmGKd7tPu6W') -> bool:
    try:

        result = parsing_token(token_string, key)

        if result is None:
            return False

        if result.get('datetime') < datetime_now():  # type: ignore
            return False

        return True

    except Exception as e:
        logger.exception(e)
        return False
